package tc.oc.client;

import java.util.ArrayList;
import java.util.List;

import org.lwjgl.input.Keyboard;

import net.minecraft.client.Minecraft;
import net.minecraft.src.BaseMod;
import net.minecraft.src.GuiScreen;
import net.minecraft.src.KeyBinding;
import net.minecraft.src.ModLoader;
import net.minecraft.src.NetClientHandler;
import net.minecraft.src.NetServerHandler;
import net.minecraft.src.Packet250CustomPayload;
import tc.oc.client.data.KeyBindings;
import tc.oc.client.data.Strings;
import tc.oc.client.modules.*;
import tc.oc.client.modules.gui.*;
import tc.oc.client.modules.notifier.GUINotifier;
import tc.oc.client.modules.stats.MainStats;
import tc.oc.client.util.UCLogger;

public class UndercastClient {

    // Version control (based on minecraft forge's version control)
    // This number is incremented every time we remove deprecated code/major API changes, never reset
    public static final int majorVersion = 0;
    // This number is incremented every minecraft release, never reset
    public static final int minorVersion = 0;
    // This number is incremented every time a interface changes or new major feature is added, and reset every Minecraft version
    public static final int revisionVersion = 2;

    private BaseMod main_mod;
    private ArrayList<IModule> modules = new ArrayList();
    private KeyBindings key_binder;

    public UndercastClient(BaseMod main_mod) {
        // Update variables
        this.main_mod = main_mod;
        
        // TODO: Config

        // Key Bindings
        key_binder = new KeyBindings(main_mod);

        // Add modules
        modules.add(new GUIMainMenu(this));
        modules.add(new GUIDeathMenu(this));
        modules.add(new GUINotifier(this));
        modules.add(new MainStats(this));

        // Default to all active; replace with reading in from config
        for(IModule temp:modules){
            temp.setState(true);
        }

        // Display to the user what is active and what is not
        for(IModule temp:modules){
            if(temp.getState())
                UCLogger.logInfo("Active= " + temp.getName() + " (" + temp.getVersion() + ")");
            else
                UCLogger.logInfo("Inactive= " + temp.getName() + " (" + temp.getVersion() + ")");
        }

        // Done
        UCLogger.logInfo("Mod Loaded!");
    }

    /**
     * This is called on server join This method sets if the user is on Project Ares or not
     */
    public void clientConnect(NetClientHandler handler) {
        for(IModule temp:modules){
            if(temp.getState() && temp instanceof IServerConnect)
                ((IServerConnect) temp).clientConnect(handler);
        }
    }

    /**
     * This Method is called when the client disconnects from a server; It is also called when the program is exited from ingame
     */
    public void clientDisconnect(NetClientHandler handler) {
        for(IModule temp:modules){
            if(temp.getState() && temp instanceof IServerDisconnect)
                ((IServerDisconnect) temp).clientDisconnect(handler);
        }
    }

    /**
     * On a game tick this method is called; It is important to include multi-threading
     */
    public boolean onTickInGame(float tick, Minecraft mc) {
        for(IModule temp:modules){
            if(temp.getState() && temp instanceof IGameTick)
                ((IGameTick) temp).onTickInGame(tick, mc);
        }
        return true;
    }

    /**
     * This Method is called every tick a gui is open
     * 
     * @param screen the current screen shown
     */
    public boolean onTickInGUI(float tick, Minecraft mc, GuiScreen screen) {
        for(IModule temp:modules){
            if(temp.getState() && temp instanceof IGuiTick)
                ((IGuiTick) temp).onTickInGUI(tick, mc, screen);
        }
        return true;
    }

    /**
     * Called when a key that has been registered has been pressed.
     * 
     * @param key the key binding of what the user has pressed
     */
    public void keyboardEvent(KeyBinding key) {
        for(IModule temp:modules){
            if(temp.getState() && temp instanceof IKeyboard)
                ((IKeyboard) temp).keyboardEvent(key);
        }
    }

    /**
     * A event for getting packet data from the server
     */
    public void serverCustomPayload(NetServerHandler serverHandler, Packet250CustomPayload packet) {
        for(IModule temp:modules){
            if(temp.getState() && temp instanceof IServerPacket)
                ((IServerPacket) temp).serverCustomPayload(serverHandler, packet);
        }
    }

    /**
     * Every time a client receives a chat line this method is called.
     */
    public void clientChat(String var1) {
        for(IModule temp:modules){
            if(temp.getState() && temp instanceof IClientChat)
                ((IClientChat) temp).clientChat(var1);
        }
    }
}
