package tc.oc.client.modules.stats;

import net.minecraft.client.Minecraft;
import net.minecraft.src.GuiChat;
import net.minecraft.src.GuiGameOver;
import net.minecraft.src.GuiScreen;
import net.minecraft.src.KeyBinding;
import net.minecraft.src.NetClientHandler;
import tc.oc.client.UndercastClient;
import tc.oc.client.data.KeyBindings;
import tc.oc.client.data.Strings;
import tc.oc.client.data.Teams;
import tc.oc.client.modules.IClientChat;
import tc.oc.client.modules.IGameTick;
import tc.oc.client.modules.IGuiTick;
import tc.oc.client.modules.IKeyboard;
import tc.oc.client.modules.IModule;
import tc.oc.client.modules.IServerConnect;
import tc.oc.client.util.UCLogger;

public class MainStats implements IModule,IGameTick,IServerConnect,IClientChat,IKeyboard {
    private UndercastClient client;
    private String name = "Player Stats";
    private String version = "0.1.6";
    private boolean state = false;
    private boolean gui_active = true;
    protected StatData data;

    public MainStats(UndercastClient client) {
        this.client = client;
        data = new StatData(client);
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getVersion() {
        return version;
    }

    @Override
    public void setState(boolean state) {
        this.state = state;
    }

    @Override
    public boolean getState() {
        return state;
    }

    @Override
    public boolean onTickInGame(float tick, Minecraft mc) {
        // Update the data
        data.updateData();
        // Display data
        this.updateIngameGui(tick, mc);
        // Full bright update; If not an obs. then make sure it is not active.
        if(!data.display_team.contains(Teams.Observers.toString())){
            data.fullbright_toggle = false;
            if(mc.gameSettings.gammaSetting >= data.brightLevel){
                mc.gameSettings.gammaSetting = data.defaultLevel;
            }
        }
        // If brightness settings change then update the default value.
        else if(mc.gameSettings.gammaSetting != data.brightLevel && mc.gameSettings.gammaSetting < data.brightLevel){
            data.fullbright_toggle = false;
            data.defaultLevel = mc.gameSettings.gammaSetting;
        }
        return false;
    }

    @Override
    public void clientConnect(NetClientHandler handler) {
        // Update server join
        data.servertime_login = System.currentTimeMillis();
        data.server = "Lobby";
        // Reset all stat data
        data.resetStatData();
    }

    @Override
    public void clientChat(String var1) {
        // Send chat to chat handler to update data
        StatChat.clientChat(var1, data);
    }

    @Override
    public void keyboardEvent(KeyBinding key) {
        // If an obs. and key is pressed then toggle full bright
        if(KeyBindings.fullbright_kb != null && key == KeyBindings.fullbright_kb && data.display_team.contains(Teams.Observers.toString())){
            if(Minecraft.getMinecraft().inGameHasFocus){
                data.fullbright_toggle = !data.fullbright_toggle;
                if(data.fullbright_toggle)
                    Minecraft.getMinecraft().gameSettings.gammaSetting = data.brightLevel;
                else
                    Minecraft.getMinecraft().gameSettings.gammaSetting = data.defaultLevel;
                Minecraft.getMinecraft().sndManager.playSoundFX("random.click", 0.5F, 1.0F);
            }
        }
        // If stat gui toggle key is pressed toggle the display
        if(KeyBindings.ingamestats_kb != null && key == KeyBindings.ingamestats_kb){
            gui_active = !gui_active;
        }
    }

    /**
     * A private method that displays all the InGame stats.
     */
    private void updateIngameGui(float tick, Minecraft mc) {
        // If gui is not active then don't display anything
        if(!gui_active)
            return;
        // Verify it is correct to display
        if(!(mc.inGameHasFocus) && !(mc.currentScreen instanceof GuiChat || mc.currentScreen instanceof GuiGameOver))
            return;
        // Display stats
        int height = 2;
        int width = 2;
        int space = mc.fontRenderer.FONT_HEIGHT;
        // Display fps/playing time
        mc.fontRenderer.drawStringWithShadow(Strings.COLORCODE_FPS + "[" + data.display_servertime + "] [" + data.fps + "]", width, height, 16777215);
        height += space;
        // Server
        mc.fontRenderer.drawStringWithShadow("Server: " + Strings.COLORCODE_SERVER + data.server, width, height, 16777215);
        height += space;
        // Main stats
        mc.fontRenderer.drawStringWithShadow("Team: " + data.display_team, width, height, 16777215);
        height += space;
        mc.fontRenderer.drawStringWithShadow("K/D: " + Strings.COLORCODE_KD + data.display_kd, width, height, 16777215);
        height += space;
        mc.fontRenderer.drawStringWithShadow("K/K: " + Strings.COLORCODE_KK + data.display_kk, width, height, 16777215);
        height += space;
        mc.fontRenderer.drawStringWithShadow("Kills: " + Strings.COLORCODE_KILLS + data.kills_total, width, height, 16777215);
        height += space;
        mc.fontRenderer.drawStringWithShadow("Deaths: " + Strings.COLORCODE_DEATHS + data.death_total, width, height, 16777215);
        height += space;
        mc.fontRenderer.drawStringWithShadow("Streak: " + Strings.COLORCODE_KILLSTREAK + data.killstreak_current + "/" + data.killstreak_max, width, height, 16777215);
        height += space;
        // Full bright
        if(data.display_team.contains(Teams.Observers.toString())){
            if(data.fullbright_toggle){
                mc.fontRenderer.drawStringWithShadow("Full Bright: \u00A72ON", width, height, 16777215);
                height += 8;
            }else{
                mc.fontRenderer.drawStringWithShadow("Full Bright: \u00A7cOFF", width, height, 16777215);
                height += 8;
            }
        }

    }

}
